#!/bin/sh
#
# Copyright (c) 2021 VMware, Inc. All Rights Reserved.
#
# Make heap dump and thread dump if memory pressure is greater than the percentage that is provided as an argument
#
# The first argument of the script is the percentage of memory pressure

# validate whether there is an argument passed
percentage=${1:-90}

: ${HEAP_DUMP_PATH:=/var/log}

memory_usage_in_bytes=$(cat /sys/fs/cgroup/memory/memory.usage_in_bytes)
memory_limit_in_bytes=$(cat /sys/fs/cgroup/memory/memory.limit_in_bytes)
threshold=$(((( $(cat /sys/fs/cgroup/memory/memory.limit_in_bytes) / 100 )) * $percentage))

# check whether memory pressure limit is reached
if [ ${memory_usage_in_bytes} -gt ${threshold} ]
then

  echo "$( date +'%Y-%m-%dT%H:%M:%S' ) $HOSTNAME Hit memory pressure limit. Will create memory dump. Current usage in bytes ${memory_usage_in_bytes}, limit ${memory_limit_in_bytes}, threshold ${threshold}." >> ${HEAP_DUMP_PATH}/memory_dump.log

# disable cronjob on first execution
  rm /etc/cron.d/heapAndThreadDump

# remove data from previous executions
  find ${HEAP_DUMP_PATH} -name "thread_dump_*" -exec rm {} \;
  find ${HEAP_DUMP_PATH} -name "heap_dump_*" -exec rm {} \;
  find ${HEAP_DUMP_PATH} -name "processes_*" -exec rm {} \;

# make heap dump and thread dump
  a=$(date +'%Y-%m-%d_%H-%M-%S')
  ps -aux > ${HEAP_DUMP_PATH}/processes_$a
  /jdk/bin/jps | \
  grep -iv jps | \
  awk '{print $1}' | \
  while read p
  do
    /jdk/bin/jstack -l -e $p > ${HEAP_DUMP_PATH}/thread_dump_${p}_$a.txt && \
    /jdk/bin/jmap -dump:format=b,file=${HEAP_DUMP_PATH}/heap_dump_${p}_$a.hprof $p
  done

  echo "$( date +'%Y-%m-%dT%H:%M:%S' ) $HOSTNAME Create memory dump finished" >> ${HEAP_DUMP_PATH}/memory_dump.log

fi

